part of jaguar_jwt.src;

const String kJwtHeaderKey = 'Authorization';

/// Holds all the info required for JWT token encoding and decoding
class JwtInfo {
  final String issuer;

  final List<String> audience;

  final JwaSignatureContext signatureContext;

  final Duration tokenDuration;

  const JwtInfo(this.signatureContext, this.issuer,
      {this.audience: const <String>[],
      this.tokenDuration: const Duration(days: 1)});

  JwtValidationContext validationContext() {
    return new JaguarJwtValidationContext(signatureContext,
        new JaguarJwtClaimSetValidationContext(issuer, audience));
  }
}

/// Encodes, decodes and validates JWT tokens
class JwtManager {
  final JwtInfo info;

  JwtValidationContext _validationContext;

  JwtManager(this.info) {
    _validationContext = info.validationContext();
  }

  /// Issues custom token
  String encodeToken(JaguarJwtClaimSet claimSet) {
    JsonWebToken jwt = new JsonWebToken.jws(claimSet, info.signatureContext);
    return jwt.encode();
  }

  /// Decodes the token
  JsonWebToken decodeToken(String token) => new JsonWebToken.decode(token,
      validationContext: _validationContext,
      claimSetParser: JaguarJwtClaimSet.parse);

  /// Validates token
  bool isTokenValid(String token) {
    if (token == null) {
      return false;
    }
    JsonWebToken jwt = decodeToken(token);
    JaguarJwtClaimSet claimSet = jwt.claimSet;
    Set<ConstraintViolation> violations =
        claimSet.validate(_validationContext.claimSetValidationContext);
    if (violations.isEmpty &&
        (claimSet?.audience == null || claimSet.audience.isEmpty)) {
      return true;
    }

    if (violations.isEmpty && claimSet?.audience != null) {
      for (String aud in claimSet.audience) {
        if (info.audience.contains(aud)) {
          return true;
        }
      }
    }
    return false;
  }

  bool isClaimSetValid(JaguarJwtClaimSet claimSet) {
    Set<ConstraintViolation> violations =
        claimSet.validate(_validationContext.claimSetValidationContext);

    return violations.isEmpty;
  }
}
