library test.no_payload;

import 'dart:async';
import 'dart:io';

import 'package:test/test.dart';

import 'package:jaguar/jaguar.dart';
import 'package:jaguar/testing.dart';

import 'package:auth_header/auth_header.dart';
import 'package:dart_jwt/dart_jwt.dart';

import '../../example/models/book/book.dart';
import '../../example/simple/routes/routes.dart';

part 'no_payload.g.dart';

@Api()
class Bibliotek extends _$JaguarBibliotek implements RequestHandler {
  @Group()
  final AuthManager auth = new AuthManager();

  @Group()
  final Admin admin = new Admin();

  @Group()
  final Student student = new Student();
}

main() {
  group('No payload', () {
    JaguarMock mock;

    setUp(() {
      Configuration config = new Configuration();
      config.addApi(new Bibliotek());
      mock = new JaguarMock(config);
    });

    tearDown(() {});

    test('Login', () async {
      String authHeader = "";
      {
        Uri uri = new Uri.http('localhost:8080', '/login',
            {'username': 'kleak', 'password': 'pass'});
        MockHttpRequest rq = new MockHttpRequest(uri, method: 'POST');
        MockHttpResponse response = await mock.handleRequest(rq);

        expect(response.mockContent, r'{}');
        expect(response.statusCode, 200);
        authHeader = response.headers.value(HttpHeaders.AUTHORIZATION);
        expect(authHeader, isNotNull);
        AuthHeaderItem item =
            new AuthHeaderItem.fromHeaderBySchema(authHeader, 'Bearer');
        OpenIdJwtClaimSet claimSet =
            (kJwtHelper.decodeToken(item.credentials) as JwtInJws).claimSet;
        expect(claimSet.issuer, 'library');
        expect(claimSet.subject, 'kleak');
        expect(claimSet.audience, ['student']);
      }

      {
        Uri uri = new Uri.http('localhost:8080', '/book/all');
        MockHttpRequest rq = new MockHttpRequest(uri, method: 'GET');
        rq.headers.set(HttpHeaders.AUTHORIZATION, authHeader);
        MockHttpResponse response = await mock.handleRequest(rq);

        expect(response.mockContent, r'[]');
        expect(response.statusCode, 200);
        authHeader = response.headers.value(HttpHeaders.AUTHORIZATION);
        expect(authHeader, isNotNull);
        AuthHeaderItem item =
            new AuthHeaderItem.fromHeaderBySchema(authHeader, 'Bearer');
        OpenIdJwtClaimSet claimSet =
            (kJwtHelper.decodeToken(item.credentials) as JwtInJws).claimSet;
        expect(claimSet.issuer, 'library');
        expect(claimSet.subject, 'kleak');
        expect(claimSet.audience, ['student']);
      }
    });

    test('Unauthorised error', () async {
      Uri uri = new Uri.http('localhost:8080', '/book/all');
      MockHttpRequest rq = new MockHttpRequest(uri, method: 'GET');
      MockHttpResponse response = await mock.handleRequest(rq);

      expect(response.statusCode, 401);
      /* TODO
      authHeader = response.headers.value(HttpHeaders.AUTHORIZATION);
      expect(authHeader, isNotNull);
      AuthHeaderItem item = findAuthHeaderItemByScheme(authHeader, 'Bearer');
      OpenIdJwtClaimSet claimSet =
          (kJwtHelper.decodeToken(item.credentials) as JwtInJws).claimSet;
      expect(claimSet.issuer, 'library');
      expect(claimSet.subject, 'kleak');
      expect(claimSet.audience, ['student']);
      */
    });
  });
}
